import { ApplicationDataSource } from '../database/datasource.js';
import { LtiResourceModel } from '../database/model.js';
import { LTIResourceStudent } from './lti_resource_student.js';
export class LTIResource {
    resource_model;
    resource_id;
    experiment_template_uri;
    // #region Initializers
    constructor(resource_model) {
        this.resource_model = resource_model;
        this.resource_id = resource_model.id;
        this.experiment_template_uri = resource_model.experiment_template_uri;
    }
    static async getOrCreate(message, platform) {
        const resource_init = {
            platform: { id: platform.platform_model.id },
            resource_link_id: message.resource_link_id,
        };
        await ApplicationDataSource.createQueryBuilder(LtiResourceModel, 'resource').insert().values({ ...resource_init }).orIgnore().execute();
        const resource = await ApplicationDataSource.manager.findOneByOrFail(LtiResourceModel, resource_init);
        return new LTIResource(resource);
    }
    static async byId({ resource_id }) {
        const resource = await ApplicationDataSource.manager.findOneByOrFail(LtiResourceModel, { id: resource_id });
        return new LTIResource(resource);
    }
    static async list() {
        const resources = await ApplicationDataSource.manager.find(LtiResourceModel, {});
        return resources.map(resource => new LTIResource(resource));
    }
    // #endregion
    //private async getTemplateRoles(){
    //  const template = await clients.experiment.getTemplate(this.resource_model.experiment_template_uri!)
    //  const devices = await Promise.all(template.configuration.devices.map(d=>clients.device.getDevice(d.device)))
    //  const roles: {role: string, devices: string[]}[]= []
    //  for(let i=0;i<devices.length;i++){
    //    const device = devices[i];
    //    if (device.type === 'group') {
    //      roles.push({role:template.configuration.devices[i].role, devices:device.devices.map(d=>d.url)});
    //    }
    //  }
    //  return roles;
    //}
    setNameService(context_memberships_url) {
        this.resource_model.namesServiceUrl = context_memberships_url;
        ApplicationDataSource.manager.save(LtiResourceModel, this.resource_model);
    }
    async getStudent(id) {
        return LTIResourceStudent.byId({ resource_id: this.resource_id, ...id });
    }
    async getStudents() {
        return LTIResourceStudent.list({ resource_id: this.resource_id });
    }
    /*async getStudents(): Promise<Student<'response'>[]> {
      if(!this.resource_model.namesServiceUrl){
        throw new Error("No name service provided");
      }
      const roles = await this.getTemplateRoles();
      const members = membership(this.resource_model.platform, this.resource_model.namesServiceUrl);
  
      return members.map(m=>({
        name: m.name ?? `${m.given_name} ${m.family_name}`,
        uri: this.uri+'/students/'+m.user_id,
        external_id: m.user_id,
        role_mapping: roles.map(r=>({role: r.role, device: 'GROUP'})),
      }))
    }*/
    async update(data) {
        if (data.experiment_template_uri)
            this.resource_model.experiment_template_uri = data.experiment_template_uri;
        await ApplicationDataSource.manager.save(LtiResourceModel, this.resource_model);
    }
    async delete() {
        await ApplicationDataSource.manager.delete(LtiResourceModel, this.resource_model);
    }
}
//# sourceMappingURL=lti_resource.js.map